# Get A Clue (C) 2010 V. Harishankar
# Crossword puzzle maker program
# Licensed under the GNU GPL v3

# Cross puzzle creator class
import sys

import crosswordpuzzle

class CrosswordPuzzleCreator:
	# ansi color codes for grid display
	BRICKRED = '\033[31m'
	# bold
	BOLD = '\033[33m'
	# blue
	BLUE = '\033[34m'
	# grey
	GREY = '\033[30m'
	# Black background white text
	BLACK_BG = '\033[40;1;37m'
	# white background black text
	WHITE_BG = '\033[47;0;30m'
	# white background red text
	REDWHITE_BG = '\033[47;0;31m'
	# disable colors
	ENDCOL = '\033[0m'

	def __init__ (self):
		self.do_main_loop ()
		self.current_file = None
		self.puzzle = None

	# display the grid with words
	def print_puzzle (self, no_words=False):
		# if self.puzzle is not none
		if self.puzzle:
			# get row, col and print them (with grid number if set)
			for col in range (self.puzzle.cols):
				# print the first row as column headers
				sys.stdout.write (self.BLUE + '  %02d' % col + self.ENDCOL)
			sys.stdout.write ("\n")

			for row in range (self.puzzle.rows):
				for col in range (self.puzzle.cols):
					# print the data
					# if the cell is numbered i.e. start of a word
					if self.puzzle.data[row][col].numbered != 0:
						sys.stdout.write (self.REDWHITE_BG +
						'%02d ' % self.puzzle.data[row][col].numbered + self.ENDCOL)
					# print spaces
					else:
						if self.puzzle.data[row][col].char == "#":
							sys.stdout.write (self.BLACK_BG + '   ' + self.ENDCOL)
						else:
							sys.stdout.write (self.WHITE_BG + '   ' + self.ENDCOL)
					# if the character is not a blank or a block
					if (self.puzzle.data[row][col].char <> "." and
										self.puzzle.data[row][col].char <> "#"):
						# if words are to be shown regardless of hidden/revealed state
						if no_words is False:
							sys.stdout.write (self.WHITE_BG +
								self.puzzle.data[row][col].char + self.ENDCOL)
						else:
							# display only revealed
							if self.puzzle.data[row][col].revealed is True:
								sys.stdout.write (self.WHITE_BG +
									self.puzzle.data[row][col].char + self.ENDCOL)
							# else print a blank
							else:
								sys.stdout.write (self.WHITE_BG + '.' + self.ENDCOL)
					elif self.puzzle.data[row][col].char == '.':
						sys.stdout.write (self.WHITE_BG +
								self.puzzle.data[row][col].char + self.ENDCOL)
					elif self.puzzle.data[row][col].char == '#':
						sys.stdout.write (self.BLACK_BG + " " + self.ENDCOL)

				sys.stdout.write (' ' + self.BLUE + "%2d" % row + self.ENDCOL + "\n")
			raw_input (self.BRICKRED + "Press <return> to continue" + self.ENDCOL)

	# add a word to the puzzle
	def add_word (self, across=True):
		# first display the grid
		self.print_puzzle ()
		# get the row and column
		srow = raw_input (self.BRICKRED + "Start row: " + self.ENDCOL)
		scol = raw_input (self.BRICKRED + "Start col: " + self.ENDCOL)
		# try converting it to number
		try:
			row = int (srow)
			col = int (scol)
		except ValueError:
			sys.stderr.write ("Invalid row or column\n")
			return
		# get the word
		word = raw_input (self.BRICKRED + "Word: " + self.ENDCOL)

		# try to add the word to the puzzle grid
		try:
			if across == True:
				self.puzzle.set_word_across (row, col, word)
			else:
				self.puzzle.set_word_down (row, col, word)
		except crosswordpuzzle.TooLongWordException:
			sys.stderr.write ("Word is too long to fit in the grid! Aborting.\n")
		except crosswordpuzzle.IntersectWordException:
			sys.stderr.write ("Word intersects badly with another word!\n")
		except crosswordpuzzle.FrozenGridException:
			sys.stderr.write ("Word cannot be added to a frozen puzzle.\n")

	# Puzzle loop
	def do_puzzle_loop (self):
		# there is a current file
		while True:
			if self.current_file and self.puzzle:
				sys.stdout.write (self.BOLD + "\n-----------------------------------\n")
				sys.stdout.write ("Puzzle: " + self.current_file + "\n")
				sys.stdout.write ("-----------------------------------" + self.ENDCOL + "\n")
				sys.stdout.write (self.BLUE + "1. Display grid\n")
				sys.stdout.write ("2. Add across word\n")
				sys.stdout.write ("3. Add down word\n")
				sys.stdout.write ("4. Remove across word\n")
				sys.stdout.write ("5. Remove down word\n")
				sys.stdout.write ("6. Freeze grid\n")
				sys.stdout.write ("7. Unfreeze grid\n")
				sys.stdout.write ("8. Set a clue\n")
				sys.stdout.write ("9. Remove clue\n")
				sys.stdout.write ("S. Save puzzle\n")
				sys.stdout.write ("X. Exit to main menu\n" + self.ENDCOL)
				ch = raw_input (self.BRICKRED + "Your choice: " + self.ENDCOL)
				if ch == "1":
					self.print_puzzle ()
				elif ch == "2":
					self.add_word ()
				elif ch == "3":
					self.add_word (False)
				elif ch == "6":
					self.puzzle.freeze_grid ()
				elif ch == "7":
					self.puzzle.unfreeze_grid ()
				elif ch == "X" or ch == "x":
					break

	# when user chooses new puzzle
	def on_new_puzzle (self):
		self.current_file = raw_input (self.BRICKRED + "New puzzle file name: "
					+ self.ENDCOL)
		srows = raw_input (self.BRICKRED + "Number of rows: " + self.ENDCOL)
		scols = raw_input (self.BRICKRED + "Number of cols: " + self.ENDCOL)
		try:
			rows = int (srows)
			cols = int (scols)
		except ValueError:
			sys.stderr.write ("Invalid number of rows/columns")
			return
		self.puzzle = crosswordpuzzle.CrosswordPuzzle (rows, cols)
		self.do_puzzle_loop ()

	# Main application loop
	def do_main_loop (self):
		# display the menu
		while True:
			sys.stdout.write (self.BOLD + "\n-----------------------------------\n")
			sys.stdout.write ("Get A Clue - Crossword Puzzle Maker\n")
			sys.stdout.write ("-----------------------------------\n" + self.ENDCOL)
			sys.stdout.write (self.BLUE + "1. Start a new puzzle\n")
			sys.stdout.write ("2. Open an existing puzzle\n")
			sys.stdout.write ("X. Exit\n" + self.ENDCOL)
			ch = raw_input (self.BRICKRED + "Your choice: " + self.ENDCOL)
			if ch == '1':
				self.on_new_puzzle ()
			if ch == 'x' or ch == 'X':
				break
